@@ -13,6 +13,7 @@ from xpander_sdk import (
     LLMProvider,
     SourceNodeType,
     GraphItem,
+    AgentGraphItemType
 )
 from ...generic import get_sub_agent_id_from_oas_by_name, wait
 from .base import SDKAdapter
@@ -523,7 +524,7 @@ class ChainlitAdapter(SDKAdapter):
         
         return tool_calls
 
-    async def process_tool_calls(self,run_id: str, tool_calls: Dict[int, ChainlitToolCallWithLLMResponse], is_sequence_agent: bool = False, sequence_agent_graph_items:List[GraphItem] = [], reported_sequence_sub_agent_ids: set = set(), on_node_enter: Optional[Callable] = None, on_node_error: Optional[Callable] = None, agent_custom_functions: list[GraphItem] = [], local_tools: Dict[str, Callable] = None) -> str | None:
+    async def process_tool_calls(self,run_id: str, tool_calls: Dict[int, ChainlitToolCallWithLLMResponse], is_sequence_agent: bool = False, sequence_agent_graph_items:List[GraphItem] = [], reported_sequence_sub_agent_ids: set = set(), on_node_enter: Optional[Callable] = None, on_node_error: Optional[Callable] = None, check_if_agent_has_dedicated_workers: Optional[Callable] = None, agent_custom_functions: list[GraphItem] = [], local_tools: Dict[str, Callable] = None) -> str | None:
         """
         Process tool calls by formatting their payloads and executing them.
 
@@ -550,10 +551,10 @@ class ChainlitAdapter(SDKAdapter):
         # Count tool calls per execution
         execution_id = self.agent.execution.id
 
-        if execution_id in self.execution_tool_calls:
+        # if execution_id in self.execution_tool_calls:
             # Check max tool call count
-            if len(self.execution_tool_calls[execution_id]) >= MAX_TOOL_CALLS:
-                is_max_tool_call_reached = True
+            # if len(self.execution_tool_calls[execution_id]) >= MAX_TOOL_CALLS:
+                # is_max_tool_call_reached = True
                 
         self.agent.add_messages(
             messages=[
@@ -619,9 +620,14 @@ class ChainlitAdapter(SDKAdapter):
             should_send_step = True
             
             tool_agent_id = get_sub_agent_id_from_oas_by_name(agent_name=tool_call.name,oas=self.agent.oas)
+            is_sub_agent_execution = False
             if tool_agent_id:
                 sub_agent_node = self.agent.graph.find_node_by_item_id(item_id=tool_agent_id)
                 if sub_agent_node:
+                    is_sub_agent_execution = True
+                    if not graph_item:
+                        graph_item = sub_agent_node
+                    
                     if on_node_enter:
                         await on_node_enter(tool_call=tool_call,graph_item=sub_agent_node)
                     tool_name = sub_agent_node.name
@@ -646,10 +652,13 @@ class ChainlitAdapter(SDKAdapter):
             
             if is_debug_mode_active:
                 current_step.name = tool_name
-                current_step.input = tool_call.payload
+                if is_sub_agent_execution:
+                    current_step.input = tool_call.payload['bodyParams']['input_task']
+                else:
+                    current_step.input = tool_call.payload
             
             if is_sub_agent_root:
-                current_step.show_input = False
+                current_step.show_input = True
                 current_step.default_open = True
                 current_step.output = None
             
@@ -714,6 +723,13 @@ class ChainlitAdapter(SDKAdapter):
                     raise Exception(error)
                 else: # execute the tool
                     tool_call_result = await make_async(self.agent.run_tool)(tool=tool_call, payload_extension={"headers":{"x-xpander-source-node-type": SourceNodeType.ASSISTANT.value.lower()}})
+                    
+                    # check if has dedicated worker and wait for result. let the user refresh the result
+                    if graph_item and graph_item.type.value == AgentGraphItemType.AGENT.value and check_if_agent_has_dedicated_workers:
+                        has_dedicated_workers = await check_if_agent_has_dedicated_workers(agent_id=graph_item.item_id,organization_id=self.agent.organization_id)
+                        if has_dedicated_workers:
+                            return "waiting_for_sub_agent_result"
+                    
             
             current_step.end = utc_now()
             
